@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,18 +26,16 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of web monitoring');
 $page['file'] = 'httpconf.php';
-$page['scripts'] = ['class.cviewswitcher.js'];
+$page['scripts'] = ['class.tagfilteritem.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'groupid'			=> [T_ZBX_INT, O_OPT, P_SYS,	DB_ID,				null],
 	'new_httpstep'		=> [T_ZBX_STR, O_OPT, P_NO_TRIM,	null,				null],
 	'group_httptestid'	=> [T_ZBX_INT, O_OPT, null,	DB_ID,				null],
 	// form
 	'hostid'          => [T_ZBX_INT, O_OPT, P_SYS, DB_ID.NOT_ZERO,          'isset({form}) || isset({add}) || isset({update})'],
-	'applicationid'   => [T_ZBX_INT, O_OPT, null,  DB_ID,                   null, _('Application')],
 	'httptestid'      => [T_ZBX_INT, O_NO,  P_SYS, DB_ID,                   'isset({form}) && {form} == "update"'],
 	'name'            => [T_ZBX_STR, O_OPT, null,  NOT_EMPTY,               'isset({add}) || isset({update})', _('Name')],
 	'delay'           => [T_ZBX_STR, O_OPT, null,  null,					'isset({add}) || isset({update})'],
@@ -51,13 +49,31 @@ $fields = [
 	],
 	'pairs'           => [T_ZBX_STR, O_OPT, P_NO_TRIM,  null,                    null],
 	'steps'           => [T_ZBX_STR, O_OPT, P_NO_TRIM,  null,                    'isset({add}) || isset({update})', _('Steps')],
-	'authentication'  => [T_ZBX_INT, O_OPT, null,  IN('0,1,2'),             'isset({add}) || isset({update})'],
-	'http_user'       => [T_ZBX_STR, O_OPT, null,  NOT_EMPTY,               '(isset({add}) || isset({update})) && isset({authentication}) && ({authentication} == '.HTTPTEST_AUTH_BASIC.
-		' || {authentication} == '.HTTPTEST_AUTH_NTLM.')', _('User')],
-	'http_password'		=> [T_ZBX_STR, O_OPT, P_NO_TRIM,	NOT_EMPTY,		'(isset({add}) || isset({update})) && isset({authentication}) && ({authentication} == '.HTTPTEST_AUTH_BASIC.
-		' || {authentication} == '.HTTPTEST_AUTH_NTLM.')', _('Password')],
+	'authentication' =>		[T_ZBX_INT, O_OPT, null,
+								IN([HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM, HTTPTEST_AUTH_KERBEROS,
+									HTTPTEST_AUTH_DIGEST
+								]),
+								'isset({add}) || isset({update})'
+							],
+	'http_user' =>			[T_ZBX_STR, O_OPT, null,  null,
+								'(isset({add}) || isset({update})) && isset({authentication})'.
+									' && ({authentication}=='.HTTPTEST_AUTH_BASIC.
+										' || {authentication}=='.HTTPTEST_AUTH_NTLM.
+										' || {authentication}=='.HTTPTEST_AUTH_KERBEROS.
+										' || {authentication} == '.HTTPTEST_AUTH_DIGEST.
+									')',
+								_('User')
+							],
+	'http_password' =>		[T_ZBX_STR, O_OPT, P_NO_TRIM, null,
+								'(isset({add}) || isset({update})) && isset({authentication})'.
+									' && ({authentication}=='.HTTPTEST_AUTH_BASIC.
+										' || {authentication}=='.HTTPTEST_AUTH_NTLM.
+										' || {authentication}=='.HTTPTEST_AUTH_KERBEROS.
+										' || {authentication} == '.HTTPTEST_AUTH_DIGEST.
+									')',
+								_('Password')
+							],
 	'http_proxy'		=> [T_ZBX_STR, O_OPT, null,	null,				'isset({add}) || isset({update})'],
-	'new_application'	=> [T_ZBX_STR, O_OPT, null,	null,				null],
 	'hostname'			=> [T_ZBX_STR, O_OPT, null,	null,				null],
 	'templated'			=> [T_ZBX_STR, O_OPT, null,	null,				null],
 	'verify_host'		=> [T_ZBX_STR, O_OPT, null,	null,				null],
@@ -65,12 +81,19 @@ $fields = [
 	'ssl_cert_file'		=> [T_ZBX_STR, O_OPT, null, null,					'isset({add}) || isset({update})'],
 	'ssl_key_file'		=> [T_ZBX_STR, O_OPT, null, null,					'isset({add}) || isset({update})'],
 	'ssl_key_password'	=> [T_ZBX_STR, O_OPT, P_NO_TRIM, null,				'isset({add}) || isset({update})'],
+	'context' =>			[T_ZBX_STR, O_MAND, P_SYS,	IN('"host", "template"'),	null],
+	'tags'				=> [T_ZBX_STR, O_OPT, null,	null,				null],
+	'show_inherited_tags' => [T_ZBX_INT, O_OPT, null, IN([0,1]),		null],
 	// filter
 	'filter_set' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'filter_rst' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'filter_status' =>		[T_ZBX_INT, O_OPT, null,
 		IN([-1, HTTPTEST_STATUS_ACTIVE, HTTPTEST_STATUS_DISABLED]), null
 	],
+	'filter_groupids'	=> [T_ZBX_INT, O_OPT, null,	DB_ID,			null],
+	'filter_hostids'	=> [T_ZBX_INT, O_OPT, null,	DB_ID,			null],
+	'filter_evaltype'	=> [T_ZBX_INT, O_OPT, null, IN([TAG_EVAL_TYPE_AND_OR, TAG_EVAL_TYPE_OR]), null],
+	'filter_tags'		=> [T_ZBX_STR, O_OPT, null,	null,			null],
 	// actions
 	'action'			=> [T_ZBX_STR, O_OPT, P_SYS|P_ACT,
 								IN('"httptest.massclearhistory","httptest.massdelete","httptest.massdisable",'.
@@ -100,36 +123,32 @@ if (!empty($_REQUEST['steps'])) {
 /*
  * Permissions
  */
-//*
-if (isset($_REQUEST['httptestid']) || !empty($_REQUEST['group_httptestid'])) {
-	$testIds = [];
-	if (isset($_REQUEST['httptestid'])) {
-		$testIds[] = $_REQUEST['httptestid'];
-	}
-	if (!empty($_REQUEST['group_httptestid'])) {
-		$testIds = array_merge($testIds, $_REQUEST['group_httptestid']);
-	}
-	if ($testIds) {
-		$testIds = array_unique($testIds);
+if (hasRequest('httptestid')) {
+	$httptests = API::HttpTest()->get([
+		'output' => [],
+		'httptestids' => getRequest('httptestid'),
+		'editable' => true
+	]);
 
-		$count = API::HttpTest()->get([
-			'countOutput' => true,
-			'httptestids' => $testIds,
-			'editable' => true
-		]);
-
-		if ($count != count($testIds)) {
-			access_deny();
-		}
+	if (!$httptests) {
+		access_deny();
 	}
 }
-if (getRequest('hostid') && !isWritableHostTemplates([getRequest('hostid')])) {
+elseif (getRequest('hostid') && !isWritableHostTemplates([getRequest('hostid')])) {
 	access_deny();
 }
 
-$groupId = getRequest('groupid');
-if ($groupId && !API::HostGroup()->get(['groupids' => $groupId])) {
-	access_deny();
+$tags = getRequest('tags', []);
+foreach ($tags as $key => $tag) {
+	if ($tag['tag'] === '' && $tag['value'] === '') {
+		unset($tags[$key]);
+	}
+	elseif (array_key_exists('type', $tag) && !($tag['type'] & ZBX_PROPERTY_OWN)) {
+		unset($tags[$key]);
+	}
+	else {
+		unset($tags[$key]['type']);
+	}
 }
 
 /*
@@ -165,35 +184,7 @@ elseif (isset($_REQUEST['clone']) && isset($_REQUEST['httptestid'])) {
 	$_REQUEST['form'] = 'clone';
 }
 elseif (hasRequest('del_history') && hasRequest('httptestid')) {
-	$result = true;
-
-	$httpTestId = getRequest('httptestid');
-
-	$httpTests = API::HttpTest()->get([
-		'output' => ['name'],
-		'httptestids' => [$httpTestId],
-		'selectHosts' => ['name'],
-		'editable' => true
-	]);
-
-	if ($httpTests) {
-		DBstart();
-
-		$result = deleteHistoryByHttpTestIds([$httpTestId]);
-		$result = ($result && DBexecute('UPDATE httptest SET nextcheck=0 WHERE httptestid='.zbx_dbstr($httpTestId)));
-
-		if ($result) {
-			$httpTest = reset($httpTests);
-			$host = reset($httpTest['hosts']);
-
-			add_audit(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_SCENARIO,
-				_('Web scenario').' ['.$httpTest['name'].'] ['.$httpTestId.'] '.
-					_('Host').' ['.$host['name'].'] '._('History cleared')
-			);
-		}
-
-		$result = DBend($result);
-	}
+	$result = deleteHistoryByHttpTestIds([getRequest('httptestid')]);
 
 	show_messages($result, _('History cleared'), _('Cannot clear history'));
 }
@@ -210,12 +201,6 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	try {
 		DBstart();
 
-		$new_application = getRequest('new_application');
-
-		if (!empty($_REQUEST['applicationid']) && $new_application) {
-			throw new Exception(_('Cannot create new application, web scenario is already assigned to application.'));
-		}
-
 		$steps = getRequest('steps', []);
 		$field_names = ['headers', 'variables', 'post_fields', 'query_fields'];
 		$i = 1;
@@ -262,7 +247,6 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			'hostid' => $_REQUEST['hostid'],
 			'name' => $_REQUEST['name'],
 			'authentication' => $_REQUEST['authentication'],
-			'applicationid' => getRequest('applicationid', 0),
 			'delay' => getRequest('delay', DB::getDefault('httptest', 'delay')),
 			'retries' => $_REQUEST['retries'],
 			'status' => hasRequest('status') ? HTTPTEST_STATUS_ACTIVE : HTTPTEST_STATUS_DISABLED,
@@ -277,7 +261,8 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			'ssl_cert_file' => getRequest('ssl_cert_file'),
 			'ssl_key_file' => getRequest('ssl_key_file'),
 			'ssl_key_password' => getRequest('ssl_key_password'),
-			'headers' => []
+			'headers' => [],
+			'tags' => $tags
 		];
 
 		foreach (getRequest('pairs', []) as $pair) {
@@ -297,39 +282,6 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		}
 		unset($variable);
 
-		if ($new_application) {
-			$exApp = API::Application()->get([
-				'output' => ['applicationid', 'flags'],
-				'hostids' => $_REQUEST['hostid'],
-				'filter' => ['name' => $new_application]
-			]);
-
-			/*
-			 * If application exists and it is a discovered application, prevent adding it to web scenario. If it is
-			 * a normal application, assign it to web scenario. Otherwise create new application.
-			 */
-			if ($exApp) {
-				if ($exApp[0]['flags'] == ZBX_FLAG_DISCOVERY_CREATED) {
-					throw new Exception(_s('Application "%1$s" already exists.', $new_application));
-				}
-				else {
-					$httpTest['applicationid'] = $exApp[0]['applicationid'];
-				}
-			}
-			else {
-				$result = API::Application()->create([
-					'name' => $new_application,
-					'hostid' => $_REQUEST['hostid']
-				]);
-				if ($result) {
-					$httpTest['applicationid'] = reset($result['applicationids']);
-				}
-				else {
-					throw new Exception(_s('Cannot add new application "%1$s".', $new_application));
-				}
-			}
-		}
-
 		if (isset($_REQUEST['httptestid'])) {
 			// unset fields that did not change
 			$dbHttpTest = API::HttpTest()->get([
@@ -340,7 +292,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			$dbHttpTest = reset($dbHttpTest);
 			$dbHttpSteps = zbx_toHash($dbHttpTest['steps'], 'httpstepid');
 
-			$httpTest = CArrayHelper::unsetEqualValues($httpTest, $dbHttpTest, ['applicationid']);
+			$httpTest = CArrayHelper::unsetEqualValues($httpTest, $dbHttpTest);
 			foreach (['headers', 'variables'] as $field_name) {
 				if (count($httpTest[$field_name]) !== count($dbHttpTest[$field_name])) {
 					continue;
@@ -361,6 +313,10 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			}
 
 			foreach ($httpTest['steps'] as $snum => $step) {
+				if ($step['httpstepid'] < 1) {
+					unset($step['httpstepid']);
+					unset($httpTest['steps'][$snum]['httpstepid']);
+				}
 				if (array_key_exists('httpstepid', $step) && array_key_exists($step['httpstepid'], $dbHttpSteps)) {
 					$db_step = $dbHttpSteps[$step['httpstepid']];
 					$new_step = CArrayHelper::unsetEqualValues($step, $db_step, ['httpstepid']);
@@ -389,8 +345,10 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			}
 
 			$httpTest['httptestid'] = $httpTestId = $_REQUEST['httptestid'];
+
 			$result = API::HttpTest()->update($httpTest);
 			if (!$result) {
+
 				throw new Exception();
 			}
 			else {
@@ -420,6 +378,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	catch (Exception $e) {
 		DBend(false);
 
+
 		$msg = $e->getMessage();
 		if (!empty($msg)) {
 			error($msg);
@@ -463,42 +422,12 @@ elseif (hasRequest('action') && str_in_array(getRequest('action'), ['httptest.ma
 	show_messages($result, $messageSuccess, $messageFailed);
 }
 elseif (hasRequest('action') && getRequest('action') === 'httptest.massclearhistory'
-		&& hasRequest('group_httptestid') && is_array(getRequest('group_httptestid'))) {
-	$result = false;
+		&& hasRequest('group_httptestid') && is_array(getRequest('group_httptestid'))
+		&& getRequest('group_httptestid')) {
+	$result = deleteHistoryByHttpTestIds(getRequest('group_httptestid'));
 
-	$httpTestIds = getRequest('group_httptestid');
-
-	$httpTests = API::HttpTest()->get([
-		'output' => ['httptestid', 'name'],
-		'httptestids' => $httpTestIds,
-		'selectHosts' => ['name'],
-		'editable' => true
-	]);
-
-	if ($httpTests) {
-		DBstart();
-
-		$result = deleteHistoryByHttpTestIds($httpTestIds);
-		$result = ($result && DBexecute(
-			'UPDATE httptest SET nextcheck=0 WHERE '.dbConditionInt('httptestid', $httpTestIds)
-		));
-
-		if ($result) {
-			foreach ($httpTests as $httpTest) {
-				$host = reset($httpTest['hosts']);
-
-				add_audit(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_SCENARIO,
-					_('Web scenario').' ['.$httpTest['name'].'] ['.$httpTest['httptestid'].'] '.
-						_('Host').' ['.$host['name'].'] '._('History cleared')
-				);
-			}
-		}
-
-		$result = DBend($result);
-
-		if ($result) {
-			uncheckTableRows(getRequest('hostid'));
-		}
+	if ($result) {
+		uncheckTableRows(getRequest('hostid'));
 	}
 
 	show_messages($result, _('History cleared'), _('Cannot clear history'));
@@ -513,7 +442,15 @@ elseif (hasRequest('action') && getRequest('action') === 'httptest.massdelete'
 	show_messages($result, _('Web scenario deleted'), _('Cannot delete web scenario'));
 }
 
-show_messages();
+if (hasRequest('action') && hasRequest('group_httptestid') && !$result) {
+	$httptests = API::HttpTest()->get([
+		'output' => [],
+		'httptestids' => getRequest('group_httptestid'),
+		'editable' => true
+	]);
+
+	uncheckTableRows(getRequest('hostid'), zbx_objectValues($httptests, 'httptestid'));
+}
 
 /*
  * Display
@@ -524,38 +461,45 @@ if (isset($_REQUEST['form'])) {
 		'httptestid' => getRequest('httptestid'),
 		'form' => getRequest('form'),
 		'form_refresh' => getRequest('form_refresh'),
-		'templates' => []
+		'templates' => [],
+		'context' => getRequest('context'),
+		'show_inherited_tags' => getRequest('show_inherited_tags', 0)
 	];
 
 	$host = API::Host()->get([
 		'output' => ['status'],
+		'selectParentTemplates' => ['templateid', 'name'],
 		'hostids' => $data['hostid'],
 		'templated_hosts' => true
 	]);
 	$data['host'] = reset($host);
 
-	if (hasRequest('httptestid') && !hasRequest('form_refresh')) {
+	if (hasRequest('httptestid')) {
 		$db_httptests = API::HttpTest()->get([
-			'output' => ['name', 'applicationid', 'delay', 'retries', 'status', 'agent', 'authentication',
+			'output' => ['name', 'delay', 'retries', 'status', 'agent', 'authentication',
 				'http_user', 'http_password', 'http_proxy', 'templateid', 'verify_peer', 'verify_host', 'ssl_cert_file',
 				'ssl_key_file', 'ssl_key_password', 'headers', 'variables'
 			],
 			'selectSteps' => ['httpstepid', 'name', 'no', 'url', 'timeout', 'posts', 'required', 'status_codes',
 				'follow_redirects', 'retrieve_mode', 'headers', 'variables', 'query_fields', 'post_type'
 			],
+			'selectTags' => ['tag', 'value'],
 			'httptestids' => getRequest('httptestid')
 		]);
 
 		$db_httptest = $db_httptests[0];
+	}
+	else {
+		$db_httptest = null;
+	}
 
+	if ($db_httptest && !hasRequest('form_refresh')) {
 		$data['name'] = $db_httptest['name'];
-		$data['applicationid'] = $db_httptest['applicationid'];
-		$data['new_application'] = '';
 		$data['delay'] = $db_httptest['delay'];
 		$data['retries'] = $db_httptest['retries'];
 		$data['status'] = $db_httptest['status'];
 		$data['templates'] = makeHttpTestTemplatesHtml($db_httptest['httptestid'],
-			getHttpTestParentTemplates($db_httptests)
+			getHttpTestParentTemplates($db_httptests), CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES)
 		);
 
 		$data['agent'] = ZBX_AGENT_OTHER;
@@ -649,11 +593,8 @@ if (isset($_REQUEST['form'])) {
 		}
 
 		$data['name'] = getRequest('name', '');
-		$data['applicationid'] = getRequest('applicationid');
-		$data['new_application'] = getRequest('new_application', '');
 		$data['delay'] = getRequest('delay', DB::getDefault('httptest', 'delay'));
 		$data['retries'] = getRequest('retries', 1);
-
 		$data['agent'] = getRequest('agent', ZBX_DEFAULT_AGENT);
 		$data['agent_other'] = getRequest('agent_other');
 
@@ -681,149 +622,282 @@ if (isset($_REQUEST['form'])) {
 		$data['pairs'] = array_values(getRequest('pairs', []));
 	}
 
-	$data['application_list'] = [];
-	if (!empty($data['hostid'])) {
-		$dbApps = DBselect(
-			'SELECT a.applicationid,a.name'.
-			' FROM applications a'.
-			' WHERE a.hostid='.zbx_dbstr($data['hostid']).
-				' AND a.flags='.ZBX_FLAG_DISCOVERY_NORMAL
-		);
-		while ($dbApp = DBfetch($dbApps)) {
-			$data['application_list'][$dbApp['applicationid']] = $dbApp['name'];
+	$i = 1;
+	foreach($data['steps'] as $stepid => $step) {
+		$pairs_grouped = [
+			'query_fields' => [],
+			'post_fields' => [],
+			'variables' => [],
+			'headers' => []
+		];
+
+		if (array_key_exists('pairs', $step)) {
+			foreach ($step['pairs'] as $field) {
+				$pairs_grouped[$field['type']][] = $field;
+			}
+			$data['steps'][$stepid]['pairs'] = $pairs_grouped;
 		}
+		$data['steps'][$stepid]['no'] = $i++;
+	}
+
+	if ($db_httptest) {
+		$parent_templates = getHttpTestParentTemplates($db_httptests)['templates'];
+
+		$rw_templates = $data['host']['parentTemplates']
+			? API::Template()->get([
+				'output' => [],
+				'templateids' => array_keys($parent_templates),
+				'editable' => true,
+				'preservekeys' => true
+			])
+			: [];
+
+		foreach ($parent_templates as $templateid => &$template) {
+			if (array_key_exists($templateid, $rw_templates)) {
+				$template['permission'] = PERM_READ_WRITE;
+			}
+			else {
+				$template['name'] = _('Inaccessible template');
+				$template['permission'] = PERM_DENY;
+			}
+		}
+		unset($template);
+
+		$tags = getHttpTestTags([
+			'templates' => $parent_templates,
+			'hostid' => getRequest('hostid'),
+			'tags' => hasRequest('form_refresh') ? $tags : $db_httptest['tags'],
+			'show_inherited_tags' => $data['show_inherited_tags']
+		]);
+	}
+
+	$data['tags'] = $tags;
+	if (!$data['tags']) {
+		$data['tags'][] = ['tag' => '', 'value' => ''];
+	}
+	else {
+		CArrayHelper::sort($data['tags'], ['tag', 'value']);
 	}
 
 	// render view
-	$httpView = new CView('configuration.httpconf.edit', $data);
-	$httpView->render();
-	$httpView->show();
+	echo (new CView('configuration.httpconf.edit', $data))->getOutput();
 }
 else {
-	$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'name'));
-	$sortOrder = getRequest('sortorder', CProfile::get('web.'.$page['file'].'.sortorder', ZBX_SORT_UP));
+	$data = [
+		'context' => getRequest('context')
+	];
 
-	CProfile::update('web.'.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
-	CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
+	$prefix = ($data['context'] === 'host') ? 'web.hosts.' : 'web.templates.';
+
+	$sortField = getRequest('sort', CProfile::get($prefix.$page['file'].'.sort', 'name'));
+	$sortOrder = getRequest('sortorder', CProfile::get($prefix.$page['file'].'.sortorder', ZBX_SORT_UP));
+
+	CProfile::update($prefix.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
+	CProfile::update($prefix.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
 
 	if (hasRequest('filter_set')) {
-		CProfile::update('web.httpconf.filter_status', getRequest('filter_status', -1), PROFILE_TYPE_INT);
+		CProfile::update($prefix.'httpconf.filter_status', getRequest('filter_status', -1), PROFILE_TYPE_INT);
+		CProfile::updateArray($prefix.'httpconf.filter_groupids', getRequest('filter_groupids', []), PROFILE_TYPE_ID);
+		CProfile::updateArray($prefix.'httpconf.filter_hostids', getRequest('filter_hostids', []), PROFILE_TYPE_ID);
+		CProfile::update($prefix.'httpconf.filter.evaltype', getRequest('filter_evaltype', TAG_EVAL_TYPE_AND_OR),
+			PROFILE_TYPE_INT
+		);
+
+		$filter_tags_fmt = [
+			'tags' => [],
+			'values' => [],
+			'operators' => []
+		];
+
+		foreach (getRequest('filter_tags', []) as $filter_tag) {
+			if ($filter_tag['tag'] === '' && $filter_tag['value'] === '') {
+				continue;
+			}
+
+			$filter_tags_fmt['tags'][] = $filter_tag['tag'];
+			$filter_tags_fmt['values'][] = $filter_tag['value'];
+			$filter_tags_fmt['operators'][] = $filter_tag['operator'];
+		}
+
+		CProfile::updateArray($prefix.'httpconf.filter.tags.tag', $filter_tags_fmt['tags'], PROFILE_TYPE_STR);
+		CProfile::updateArray($prefix.'httpconf.filter.tags.value', $filter_tags_fmt['values'], PROFILE_TYPE_STR);
+		CProfile::updateArray($prefix.'httpconf.filter.tags.operator', $filter_tags_fmt['operators'], PROFILE_TYPE_INT);
 	}
 	elseif (hasRequest('filter_rst')) {
-		CProfile::delete('web.httpconf.filter_status');
+		CProfile::delete($prefix.'httpconf.filter_status');
+		CProfile::deleteIdx($prefix.'httpconf.filter_groupids');
+
+		$filter_hostids = getRequest('filter_hostids', CProfile::getArray($prefix.'httpconf.filter_hostids', []));
+		if (count($filter_hostids) != 1) {
+			CProfile::deleteIdx($prefix.'httpconf.filter_hostids');
+		}
+		CProfile::deleteIdx($prefix.'httpconf.filter.evaltype');
+		CProfile::deleteIdx($prefix.'httpconf.filter.tags.tag');
+		CProfile::deleteIdx($prefix.'httpconf.filter.tags.value');
+		CProfile::deleteIdx($prefix.'httpconf.filter.tags.operator');
 	}
 
-	$pageFilter = new CPageFilter([
-		'groups' => [
-			'editable' => true
-		],
-		'hosts' => [
-			'editable' => true,
-			'templated_hosts' => true
-		],
-		'hostid' => getRequest('hostid'),
-		'groupid' => getRequest('groupid')
-	]);
+	$filter = [
+		'status' => CProfile::get($prefix.'httpconf.filter_status', -1),
+		'groups' => CProfile::getArray($prefix.'httpconf.filter_groupids', null),
+		'hosts' => CProfile::getArray($prefix.'httpconf.filter_hostids', null),
+		'evaltype' => CProfile::get($prefix.'httpconf.filter.evaltype', TAG_EVAL_TYPE_AND_OR),
+		'tags' => []
+	];
 
-	$data = [
-		'hostid' => $pageFilter->hostid,
-		'pageFilter' => $pageFilter,
-		'filter_status' => CProfile::get('web.httpconf.filter_status', -1),
+	foreach (CProfile::getArray($prefix.'httpconf.filter.tags.tag', []) as $i => $tag) {
+		$filter['tags'][] = [
+			'tag' => $tag,
+			'value' => CProfile::get($prefix.'httpconf.filter.tags.value', null, $i),
+			'operator' => CProfile::get($prefix.'httpconf.filter.tags.operator', null, $i)
+		];
+	}
+
+	// Get host groups.
+	$filter['groups'] = $filter['groups']
+		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
+			'groupids' => $filter['groups'],
+			'editable' => true,
+			'preservekeys' => true
+		]), ['groupid' => 'id'])
+		: [];
+
+	$filter_groupids = $filter['groups'] ? array_keys($filter['groups']) : null;
+	if ($filter_groupids) {
+		$filter_groupids = getSubGroups($filter_groupids);
+	}
+
+	if ($data['context'] === 'host') {
+		$filter['hosts'] = $filter['hosts']
+			? CArrayHelper::renameObjectsKeys(API::Host()->get([
+				'output' => ['hostid', 'name'],
+				'hostids' => $filter['hosts'],
+				'editable' => true,
+				'preservekeys' => true
+			]), ['hostid' => 'id'])
+			: [];
+	}
+	else {
+		$filter['hosts'] = $filter['hosts']
+			? CArrayHelper::renameObjectsKeys(API::Template()->get([
+				'output' => ['templateid', 'name'],
+				'templateids' => $filter['hosts'],
+				'editable' => true,
+				'preservekeys' => true
+			]), ['templateid' => 'id'])
+			: [];
+	}
+
+	$data += [
+		'hostid' => (count($filter['hosts']) == 1)
+			? reset($filter['hosts'])['id']
+			: getRequest('hostid', 0),
+		'filter' => $filter,
 		'httpTests' => [],
 		'httpTestsLastData' => [],
 		'paging' => null,
 		'sort' => $sortField,
 		'sortorder' => $sortOrder,
-		'profileIdx' => 'web.httpconf.filter',
-		'active_tab' => CProfile::get('web.httpconf.filter.active', 1)
+		'profileIdx' => $prefix.'httpconf.filter',
+		'active_tab' => CProfile::get($prefix.'httpconf.filter.active', 1)
 	];
 
-	// show the error column only for hosts
-	if (getRequest('hostid') != 0) {
-		$data['showInfoColumn'] = (bool) API::Host()->get([
-			'hostids' => getRequest('hostid'),
-			'output' => ['status']
-		]);
+	$options = [
+		'output' => ['httptestid', $sortField],
+		'selectTags' => ['tag', 'value'],
+		'hostids' => $filter['hosts'] ? array_keys($filter['hosts']) : null,
+		'groupids' => $filter_groupids,
+		'tags' => $data['filter']['tags'],
+		'evaltype' => $data['filter']['evaltype'],
+		'templated' => ($data['context'] === 'template'),
+		'editable' => true,
+		'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1,
+		'preservekeys' => true
+	];
+	if ($data['filter']['status'] != -1) {
+		$options['filter']['status'] = $data['filter']['status'];
+	}
+
+	$httpTests = API::HttpTest()->get($options);
+
+	$dbHttpTests = DBselect(
+		'SELECT ht.httptestid,ht.name,ht.delay,ht.status,ht.hostid,ht.templateid,h.name AS hostname,ht.retries,'.
+			'ht.authentication,ht.http_proxy'.
+			' FROM httptest ht'.
+			' INNER JOIN hosts h ON h.hostid=ht.hostid'.
+			' WHERE '.dbConditionInt('ht.httptestid', zbx_objectValues($httpTests, 'httptestid'))
+	);
+	$http_tests = [];
+	while ($dbHttpTest = DBfetch($dbHttpTests)) {
+		$http_tests[$dbHttpTest['httptestid']] = $dbHttpTest + [
+			'tags' => $httpTests[$dbHttpTest['httptestid']]['tags']
+		];
+	}
+
+	order_result($http_tests, $sortField, $sortOrder);
+
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
 	}
 	else {
-		$data['showInfoColumn'] = true;
+		$page_num = CPagerHelper::loadPage($page['file']);
 	}
 
-	if ($data['pageFilter']->hostsSelected) {
-		$config = select_config();
+	CPagerHelper::savePage($page['file'], $page_num);
 
-		$options = [
-			'editable' => true,
-			'output' => ['httptestid', $sortField],
-			'limit' => $config['search_limit'] + 1
-		];
-		if ($data['filter_status'] != -1) {
-			$options['filter']['status'] = $data['filter_status'];
-		}
-		if ($data['pageFilter']->hostid > 0) {
-			$options['hostids'] = $data['pageFilter']->hostid;
-		}
-		elseif ($data['pageFilter']->groupid > 0) {
-			$options['groupids'] = $data['pageFilter']->groupids;
-		}
-		$httpTests = API::HttpTest()->get($options);
+	$data['paging'] = CPagerHelper::paginate($page_num, $http_tests, $sortOrder,
+		(new CUrl('httpconf.php'))->setArgument('context', $data['context'])
+	);
 
-		$dbHttpTests = DBselect(
-			'SELECT ht.httptestid,ht.name,ht.delay,ht.status,ht.hostid,ht.templateid,h.name AS hostname,ht.retries,'.
-				'ht.authentication,ht.http_proxy,a.applicationid,a.name AS application_name'.
-				' FROM httptest ht'.
-				' INNER JOIN hosts h ON h.hostid=ht.hostid'.
-				' LEFT JOIN applications a ON a.applicationid=ht.applicationid'.
-				' WHERE '.dbConditionInt('ht.httptestid', zbx_objectValues($httpTests, 'httptestid'))
-		);
-		$httpTests = [];
-		while ($dbHttpTest = DBfetch($dbHttpTests)) {
-			$httpTests[$dbHttpTest['httptestid']] = $dbHttpTest;
-		}
+	// Get the error column data only for hosts.
+	if ($data['context'] === 'host') {
+		$httpTestsLastData = Manager::HttpTest()->getLastData(array_keys($http_tests));
 
-		order_result($httpTests, $sortField, $sortOrder);
-
-		$url = (new CUrl('httpconf.php'))
-			->setArgument('hostid', $data['hostid'])
-			->setArgument('groupid', $data['pageFilter']->groupid);
-
-		$data['paging'] = getPagingLine($httpTests, $sortOrder, $url);
-
-		if($data['showInfoColumn']) {
-			$httpTestsLastData = Manager::HttpTest()->getLastData(array_keys($httpTests));
-
-			foreach ($httpTestsLastData as $httpTestId => &$lastData) {
-				if ($lastData['lastfailedstep'] !== null) {
-					$lastData['failedstep'] = get_httpstep_by_no($httpTestId, $lastData['lastfailedstep']);
-				}
+		foreach ($httpTestsLastData as $httpTestId => &$lastData) {
+			if ($lastData['lastfailedstep'] !== null) {
+				$lastData['failedstep'] = get_httpstep_by_no($httpTestId, $lastData['lastfailedstep']);
 			}
-			unset($lastData);
-		}
-		else {
-			$httpTestsLastData = [];
 		}
+		unset($lastData);
+	}
+	else {
+		$httpTestsLastData = [];
+	}
 
-		$dbHttpSteps = DBselect(
-			'SELECT hs.httptestid,COUNT(*) AS stepscnt'.
-				' FROM httpstep hs'.
-				' WHERE '.dbConditionInt('hs.httptestid', zbx_objectValues($httpTests, 'httptestid')).
-				' GROUP BY hs.httptestid'
-		);
-		while ($dbHttpStep = DBfetch($dbHttpSteps)) {
-			$httpTests[$dbHttpStep['httptestid']]['stepscnt'] = $dbHttpStep['stepscnt'];
-		}
+	$dbHttpSteps = DBselect(
+		'SELECT hs.httptestid,COUNT(*) AS stepscnt'.
+			' FROM httpstep hs'.
+			' WHERE '.dbConditionInt('hs.httptestid', array_column($http_tests, 'httptestid')).
+			' GROUP BY hs.httptestid'
+	);
+	while ($dbHttpStep = DBfetch($dbHttpSteps)) {
+		$http_tests[$dbHttpStep['httptestid']]['stepscnt'] = $dbHttpStep['stepscnt'];
+	}
 
-		order_result($httpTests, $sortField, $sortOrder);
+	order_result($http_tests, $sortField, $sortOrder);
 
-		$data['parent_templates'] = getHttpTestParentTemplates($httpTests);
-		$data['httpTests'] = $httpTests;
-		$data['httpTestsLastData'] = $httpTestsLastData;
+	$data['parent_templates'] = getHttpTestParentTemplates($http_tests);
+	$data['http_tests'] = $http_tests;
+	$data['httpTestsLastData'] = $httpTestsLastData;
+	$data['allowed_ui_conf_templates'] = CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES);
+
+	$data['tags'] = makeTags($data['http_tests'], true, 'httptestid', ZBX_TAG_COUNT_DEFAULT);
+
+	if (!$data['filter']['tags']) {
+		$data['filter']['tags'] = [[
+			'tag' => '',
+			'value' => '',
+			'operator' => TAG_OPERATOR_LIKE
+		]];
 	}
 
 	// render view
-	$httpView = new CView('configuration.httpconf.list', $data);
-	$httpView->render();
-	$httpView->show();
+	echo (new CView('configuration.httpconf.list', $data))->getOutput();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
