@@ -1,7 +1,7 @@
-<?php declare(strict_types = 1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,11 +26,14 @@ class CFormGrid extends CTag {
 	 */
 	private const ZBX_STYLE_CLASS = 'form-grid';
 
-	public const ZBX_STYLE_FORM_GRID_3_1 = 'form-grid-3-1';
 	public const ZBX_STYLE_FORM_GRID_1_1 = 'form-grid-1-1';
+	public const ZBX_STYLE_FORM_GRID_3_1 = 'form-grid-3-1';
 
-	public function __construct() {
-		parent::__construct('div', true);
+	// True label column width for use in filter forms.
+	public const ZBX_STYLE_FORM_GRID_LABEL_WIDTH_TRUE = 'label-width-true';
+
+	public function __construct($items = null) {
+		parent::__construct('div', true, $items);
 
 		$this->addClass(self::ZBX_STYLE_CLASS);
 	}
