@@ -23,6 +23,8 @@
  * @var CView $this
  */
 
+require_once dirname(__FILE__).'/js/configuration.trigger.prototype.list.js.php';
+
 $widget = (new CWidget())
 	->setTitle(_('Trigger prototypes'))
 	->setControls(
@@ -45,8 +47,8 @@ $url = (new CUrl('trigger_prototypes.php'))
 // create form
 $triggersForm = (new CForm('post', $url))
 	->setName('triggersForm')
-	->addVar('parent_discoveryid', $data['parent_discoveryid'])
-	->addVar('context', $data['context']);
+	->addVar('parent_discoveryid', $data['parent_discoveryid'], 'form_parent_discoveryid')
+	->addVar('context', $data['context'], 'form_context');
 
 // create table
 $triggersTable = (new CTableInfo())
@@ -176,7 +178,7 @@ foreach ($data['triggers'] as $trigger) {
 
 	$triggersTable->addRow([
 		$checkBox,
-		getSeverityCell($trigger['priority']),
+		CSeverityHelper::makeSeverityCell((int) $trigger['priority']),
 		$description,
 		$trigger['opdata'],
 		(new CDiv($expression))->addClass(ZBX_STYLE_WORDWRAP),
@@ -200,7 +202,12 @@ $triggersForm->addItem([
 			],
 			'popup.massupdate.triggerprototype' => [
 				'content' => (new CButton('', _('Mass update')))
-					->onClick("return openMassupdatePopup(this, 'popup.massupdate.triggerprototype');")
+					->onClick(
+						"openMassupdatePopup('popup.massupdate.triggerprototype', {}, {
+							dialogue_class: 'modal-popup-static',
+							trigger_element: this
+						});"
+					)
 					->addClass(ZBX_STYLE_BTN_ALT)
 					->removeAttribute('id')
 			],
